# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Collections.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_ModelF_ice
import omero_SystemF_ice
import omero_RTypes_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Start of module omero
__name__ = 'omero'

# Start of module omero.api
_M_omero.api = Ice.openModule('omero.api')
__name__ = 'omero.api'

if '_t_SearchMetadata' not in _M_omero.api.__dict__:
    _M_omero.api._t_SearchMetadata = IcePy.defineDictionary('::omero::api::SearchMetadata', (), IcePy._t_string, _M_omero.model._t_Annotation)

if '_t_StringSet' not in _M_omero.api.__dict__:
    _M_omero.api._t_StringSet = IcePy.defineSequence('::omero::api::StringSet', (), IcePy._t_string)

if '_t_LongList' not in _M_omero.api.__dict__:
    _M_omero.api._t_LongList = IcePy.defineSequence('::omero::api::LongList', (), IcePy._t_long)

if '_t_IntegerList' not in _M_omero.api.__dict__:
    _M_omero.api._t_IntegerList = IcePy.defineSequence('::omero::api::IntegerList', (), IcePy._t_int)

if '_t_SearchMetadataList' not in _M_omero.api.__dict__:
    _M_omero.api._t_SearchMetadataList = IcePy.defineSequence('::omero::api::SearchMetadataList', (), _M_omero.api._t_SearchMetadata)

if '_t_ExperimenterList' not in _M_omero.api.__dict__:
    _M_omero.api._t_ExperimenterList = IcePy.defineSequence('::omero::api::ExperimenterList', (), _M_omero.model._t_Experimenter)

if '_t_ExperimenterGroupList' not in _M_omero.api.__dict__:
    _M_omero.api._t_ExperimenterGroupList = IcePy.defineSequence('::omero::api::ExperimenterGroupList', (), _M_omero.model._t_ExperimenterGroup)

if '_t_EventList' not in _M_omero.api.__dict__:
    _M_omero.api._t_EventList = IcePy.defineSequence('::omero::api::EventList', (), _M_omero.model._t_Event)

if '_t_EventLogList' not in _M_omero.api.__dict__:
    _M_omero.api._t_EventLogList = IcePy.defineSequence('::omero::api::EventLogList', (), _M_omero.model._t_EventLog)

if '_t_AnnotationList' not in _M_omero.api.__dict__:
    _M_omero.api._t_AnnotationList = IcePy.defineSequence('::omero::api::AnnotationList', (), _M_omero.model._t_Annotation)

if '_t_SessionList' not in _M_omero.api.__dict__:
    _M_omero.api._t_SessionList = IcePy.defineSequence('::omero::api::SessionList', (), _M_omero.model._t_Session)

if '_t_IObjectList' not in _M_omero.api.__dict__:
    _M_omero.api._t_IObjectList = IcePy.defineSequence('::omero::api::IObjectList', (), _M_omero.model._t_IObject)

if '_t_ProjectList' not in _M_omero.api.__dict__:
    _M_omero.api._t_ProjectList = IcePy.defineSequence('::omero::api::ProjectList', (), _M_omero.model._t_Project)

if '_t_DatasetList' not in _M_omero.api.__dict__:
    _M_omero.api._t_DatasetList = IcePy.defineSequence('::omero::api::DatasetList', (), _M_omero.model._t_Dataset)

if '_t_ImageList' not in _M_omero.api.__dict__:
    _M_omero.api._t_ImageList = IcePy.defineSequence('::omero::api::ImageList', (), _M_omero.model._t_Image)

if '_t_LogicalChannelList' not in _M_omero.api.__dict__:
    _M_omero.api._t_LogicalChannelList = IcePy.defineSequence('::omero::api::LogicalChannelList', (), _M_omero.model._t_LogicalChannel)

if '_t_OriginalFileList' not in _M_omero.api.__dict__:
    _M_omero.api._t_OriginalFileList = IcePy.defineSequence('::omero::api::OriginalFileList', (), _M_omero.model._t_OriginalFile)

if '_t_PixelsList' not in _M_omero.api.__dict__:
    _M_omero.api._t_PixelsList = IcePy.defineSequence('::omero::api::PixelsList', (), _M_omero.model._t_Pixels)

if '_t_PixelsTypeList' not in _M_omero.api.__dict__:
    _M_omero.api._t_PixelsTypeList = IcePy.defineSequence('::omero::api::PixelsTypeList', (), _M_omero.model._t_PixelsType)

if '_t_RoiList' not in _M_omero.api.__dict__:
    _M_omero.api._t_RoiList = IcePy.defineSequence('::omero::api::RoiList', (), _M_omero.model._t_Roi)

if '_t_ScriptJobList' not in _M_omero.api.__dict__:
    _M_omero.api._t_ScriptJobList = IcePy.defineSequence('::omero::api::ScriptJobList', (), _M_omero.model._t_ScriptJob)

if '_t_ShapeList' not in _M_omero.api.__dict__:
    _M_omero.api._t_ShapeList = IcePy.defineSequence('::omero::api::ShapeList', (), _M_omero.model._t_Shape)

if '_t_ChecksumAlgorithmList' not in _M_omero.api.__dict__:
    _M_omero.api._t_ChecksumAlgorithmList = IcePy.defineSequence('::omero::api::ChecksumAlgorithmList', (), _M_omero.model._t_ChecksumAlgorithm)

if '_t_NamedValueList' not in _M_omero.api.__dict__:
    _M_omero.api._t_NamedValueList = IcePy.defineSequence('::omero::api::NamedValueList', (), _M_omero.model._t_NamedValue)

if '_t_EventContextList' not in _M_omero.api.__dict__:
    _M_omero.api._t_EventContextList = IcePy.defineSequence('::omero::api::EventContextList', (), _M_omero.sys._t_EventContext)

if '_t_BoolArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_BoolArray = IcePy.defineSequence('::omero::api::BoolArray', (), IcePy._t_bool)

if '_t_ByteArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_ByteArray = IcePy.defineSequence('::omero::api::ByteArray', (), IcePy._t_byte)

if '_t_ShortArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_ShortArray = IcePy.defineSequence('::omero::api::ShortArray', (), IcePy._t_short)

if '_t_IntegerArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_IntegerArray = IcePy.defineSequence('::omero::api::IntegerArray', (), IcePy._t_int)

if '_t_LongArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_LongArray = IcePy.defineSequence('::omero::api::LongArray', (), IcePy._t_long)

if '_t_FloatArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_FloatArray = IcePy.defineSequence('::omero::api::FloatArray', (), IcePy._t_float)

if '_t_DoubleArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_DoubleArray = IcePy.defineSequence('::omero::api::DoubleArray', (), IcePy._t_double)

if '_t_StringArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_StringArray = IcePy.defineSequence('::omero::api::StringArray', (), IcePy._t_string)

if '_t_ByteArrayArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_ByteArrayArray = IcePy.defineSequence('::omero::api::ByteArrayArray', (), _M_omero.api._t_ByteArray)

if '_t_ShortArrayArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_ShortArrayArray = IcePy.defineSequence('::omero::api::ShortArrayArray', (), _M_omero.api._t_ShortArray)

if '_t_IntegerArrayArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_IntegerArrayArray = IcePy.defineSequence('::omero::api::IntegerArrayArray', (), _M_omero.api._t_IntegerArray)

if '_t_IntegerArrayArrayArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_IntegerArrayArrayArray = IcePy.defineSequence('::omero::api::IntegerArrayArrayArray', (), _M_omero.api._t_IntegerArrayArray)

if '_t_LongArrayArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_LongArrayArray = IcePy.defineSequence('::omero::api::LongArrayArray', (), _M_omero.api._t_LongArray)

if '_t_FloatArrayArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_FloatArrayArray = IcePy.defineSequence('::omero::api::FloatArrayArray', (), _M_omero.api._t_FloatArray)

if '_t_FloatArrayArrayArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_FloatArrayArrayArray = IcePy.defineSequence('::omero::api::FloatArrayArrayArray', (), _M_omero.api._t_FloatArrayArray)

if '_t_DoubleArrayArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_DoubleArrayArray = IcePy.defineSequence('::omero::api::DoubleArrayArray', (), _M_omero.api._t_DoubleArray)

if '_t_DoubleArrayArrayArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_DoubleArrayArrayArray = IcePy.defineSequence('::omero::api::DoubleArrayArrayArray', (), _M_omero.api._t_DoubleArrayArray)

if '_t_StringArrayArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_StringArrayArray = IcePy.defineSequence('::omero::api::StringArrayArray', (), _M_omero.api._t_StringArray)

if '_t_RTypeDictArray' not in _M_omero.api.__dict__:
    _M_omero.api._t_RTypeDictArray = IcePy.defineSequence('::omero::api::RTypeDictArray', (), _M_omero._t_RTypeDict)

if '_t_LongStringMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_LongStringMap = IcePy.defineDictionary('::omero::api::LongStringMap', (), IcePy._t_long, IcePy._t_string)

if '_t_LongIntMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_LongIntMap = IcePy.defineDictionary('::omero::api::LongIntMap', (), IcePy._t_long, IcePy._t_int)

if '_t_LongByteArrayMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_LongByteArrayMap = IcePy.defineDictionary('::omero::api::LongByteArrayMap', (), IcePy._t_long, _M_omero.api._t_ByteArray)

if '_t_LongPixelsMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_LongPixelsMap = IcePy.defineDictionary('::omero::api::LongPixelsMap', (), IcePy._t_long, _M_omero.model._t_Pixels)

if '_t_IntStringMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_IntStringMap = IcePy.defineDictionary('::omero::api::IntStringMap', (), IcePy._t_int, IcePy._t_string)

if '_t_IntegerIntegerArrayMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_IntegerIntegerArrayMap = IcePy.defineDictionary('::omero::api::IntegerIntegerArrayMap', (), IcePy._t_int, _M_omero.api._t_IntegerArray)

if '_t_IntegerDoubleArrayMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_IntegerDoubleArrayMap = IcePy.defineDictionary('::omero::api::IntegerDoubleArrayMap', (), IcePy._t_int, _M_omero.api._t_DoubleArray)

if '_t_StringRTypeMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_StringRTypeMap = IcePy.defineDictionary('::omero::api::StringRTypeMap', (), IcePy._t_string, _M_omero._t_RType)

if '_t_UserMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_UserMap = IcePy.defineDictionary('::omero::api::UserMap', (), IcePy._t_string, _M_omero.model._t_Experimenter)

if '_t_OriginalFileMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_OriginalFileMap = IcePy.defineDictionary('::omero::api::OriginalFileMap', (), IcePy._t_string, _M_omero.model._t_OriginalFile)

if '_t_StringStringMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_StringStringMap = IcePy.defineDictionary('::omero::api::StringStringMap', (), IcePy._t_string, IcePy._t_string)

if '_t_StringRStringMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_StringRStringMap = IcePy.defineDictionary('::omero::api::StringRStringMap', (), IcePy._t_string, _M_omero._t_RString)

if '_t_StringStringArrayMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_StringStringArrayMap = IcePy.defineDictionary('::omero::api::StringStringArrayMap', (), IcePy._t_string, _M_omero.api._t_StringArray)

if '_t_StringLongMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_StringLongMap = IcePy.defineDictionary('::omero::api::StringLongMap', (), IcePy._t_string, IcePy._t_long)

if '_t_StringIntMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_StringIntMap = IcePy.defineDictionary('::omero::api::StringIntMap', (), IcePy._t_string, IcePy._t_int)

if 'LongPair' not in _M_omero.api.__dict__:
    _M_omero.api.LongPair = Ice.createTempClass()
    class LongPair(object):
        def __init__(self, first=0, second=0):
            self.first = first
            self.second = second

        def __hash__(self):
            _h = 0
            _h = 5 * _h + Ice.getHash(self.first)
            _h = 5 * _h + Ice.getHash(self.second)
            return _h % 0x7fffffff

        def __compare(self, other):
            if other is None:
                return 1
            elif not isinstance(other, _M_omero.api.LongPair):
                return NotImplemented
            else:
                if self.first is None or other.first is None:
                    if self.first != other.first:
                        return (-1 if self.first is None else 1)
                else:
                    if self.first < other.first:
                        return -1
                    elif self.first > other.first:
                        return 1
                if self.second is None or other.second is None:
                    if self.second != other.second:
                        return (-1 if self.second is None else 1)
                else:
                    if self.second < other.second:
                        return -1
                    elif self.second > other.second:
                        return 1
                return 0

        def __lt__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r < 0

        def __le__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r <= 0

        def __gt__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r > 0

        def __ge__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r >= 0

        def __eq__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r == 0

        def __ne__(self, other):
            r = self.__compare(other)
            if r is NotImplemented:
                return r
            else:
                return r != 0

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_LongPair)

        __repr__ = __str__

    _M_omero.api._t_LongPair = IcePy.defineStruct('::omero::api::LongPair', LongPair, (), (
        ('first', (), IcePy._t_long),
        ('second', (), IcePy._t_long)
    ))

    _M_omero.api.LongPair = LongPair
    del LongPair

if '_t_LongPairLongMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_LongPairLongMap = IcePy.defineDictionary('::omero::api::LongPairLongMap', (), _M_omero.api._t_LongPair, IcePy._t_long)

if '_t_LongPairIntMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_LongPairIntMap = IcePy.defineDictionary('::omero::api::LongPairIntMap', (), _M_omero.api._t_LongPair, IcePy._t_int)

if '_t_LongPairToStringLongMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_LongPairToStringLongMap = IcePy.defineDictionary('::omero::api::LongPairToStringLongMap', (), _M_omero.api._t_LongPair, _M_omero.api._t_StringLongMap)

if '_t_LongPairToStringIntMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_LongPairToStringIntMap = IcePy.defineDictionary('::omero::api::LongPairToStringIntMap', (), _M_omero.api._t_LongPair, _M_omero.api._t_StringIntMap)

if '_t_IdListMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_IdListMap = IcePy.defineDictionary('::omero::api::IdListMap', (), IcePy._t_string, _M_Ice._t_LongSeq)

if '_t_StringLongListMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_StringLongListMap = IcePy.defineDictionary('::omero::api::StringLongListMap', (), IcePy._t_string, _M_omero.api._t_LongList)

if '_t_BooleanLongListMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_BooleanLongListMap = IcePy.defineDictionary('::omero::api::BooleanLongListMap', (), IcePy._t_bool, _M_omero.api._t_LongList)

if '_t_BooleanIdListMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_BooleanIdListMap = IcePy.defineDictionary('::omero::api::BooleanIdListMap', (), IcePy._t_bool, _M_omero.sys._t_LongList)

if '_t_IObjectListMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_IObjectListMap = IcePy.defineDictionary('::omero::api::IObjectListMap', (), IcePy._t_string, _M_omero.api._t_IObjectList)

if '_t_LongIObjectListMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_LongIObjectListMap = IcePy.defineDictionary('::omero::api::LongIObjectListMap', (), IcePy._t_long, _M_omero.api._t_IObjectList)

if '_t_StringShapeListMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_StringShapeListMap = IcePy.defineDictionary('::omero::api::StringShapeListMap', (), IcePy._t_string, _M_omero.api._t_ShapeList)

if '_t_LongShapeListMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_LongShapeListMap = IcePy.defineDictionary('::omero::api::LongShapeListMap', (), IcePy._t_long, _M_omero.api._t_ShapeList)

if '_t_IntShapeListMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_IntShapeListMap = IcePy.defineDictionary('::omero::api::IntShapeListMap', (), IcePy._t_int, _M_omero.api._t_ShapeList)

if '_t_LongAnnotationListMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_LongAnnotationListMap = IcePy.defineDictionary('::omero::api::LongAnnotationListMap', (), IcePy._t_long, _M_omero.api._t_AnnotationList)

if '_t_IdBooleanLongListMapMap' not in _M_omero.api.__dict__:
    _M_omero.api._t_IdBooleanLongListMapMap = IcePy.defineDictionary('::omero::api::IdBooleanLongListMapMap', (), IcePy._t_long, _M_omero.api._t_BooleanLongListMap)

# End of module omero.api

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#
#   Copyright 2008 Glencoe Software, Inc. All rights reserved.
#   Use is subject to license terms supplied in LICENSE.txt
#

"""
Module which is responsible for creating rtypes from static
factory methods. Where possible, factory methods return cached values
(the fly-weight pattern) such that <code>rbool(true) == rbool(true)</code>
might hold true.

This module is meant to be kept in sync with the abstract Java class
omero.rtypes as well as the omero/rtypes.{h,cpp} files.
"""

from builtins import str
from past.builtins import basestring, long
import omero
import Ice
import IceImport
IceImport.load("omero_RTypes_ice")
IceImport.load("omero_Scripts_ice")
IceImport.load("omero_model_RTypes_ice")

sys = __import__("sys")  # Python sys


def rtype(val):
    """
    If None or an RType, return the argument itself. Otherwise,
    attempts to dispatch to the other omero.rtypes.* static methods
    to create a proper {@link RType} subclass by checking the type
    of the input. If no conversion is found, a {@link ClientError} is
    thrown.

    Note: unlike the statically typed languages, the rtype implementation
    in Python is somewhat limited by the lack of types (float v. double)
    All float-like values will produce an omero.RFloat subclass. Similar
    problems may arise with rlong and rint
    """
    if val is None:
        return None
    elif isinstance(val, omero.RType):
        return val
    elif isinstance(val, bool):
        return rbool(val)
    elif isinstance(val, int):
        return rint(val)
    elif isinstance(val, long):  # Maintains Py2 compatibility
        return rlong(val)
    elif isinstance(val, float):
        return rfloat(val)
    elif isinstance(val, basestring):
        return rstring(val)
    elif isinstance(val, omero.model.IObject):
        return robject(val)
    elif isinstance(val, omero.Internal):
        return rinternal(val)
    elif isinstance(val, list):
        return rlist(val)
    elif isinstance(val, set):
        return rset(val)
    elif isinstance(val, dict):
        return rmap(val)
    else:
        raise omero.ClientError("Cannot handle conversion from: %s"
                                % type(val))


def wrap(val, cache=None):
    """
    """
    if cache is None:
        cache = {}
    elif id(val) in cache:
        return cache[id(val)]

    if val is None:
        return None
    elif isinstance(val, (list, tuple)):
        rv = rlist()
        cache[id(val)] = rv
        for x in val:
            rv.val.append(wrap(x, cache))
    elif isinstance(val, set):
        rv = rset()
        cache[id(val)] = rv
        for x in val:
            rv.val.add(wrap(x, cache))
    elif isinstance(val, dict):
        rv = rmap()
        cache[id(val)] = rv
        for k, v in list(val.items()):
            rv.val[k] = wrap(v, cache)
        rv._validate()
    elif isinstance(val, omero.RType):
        rv = val
    else:
        rv = rtype(val)

    return rv


def unwrap(val, cache=None):
    """
    """
    if cache is None:
        cache = {}
    elif id(val) in cache:
        return cache[id(val)]

    if val is None:
        return None
    elif isinstance(val, (list, tuple)):
        rv = []
        cache[id(val)] = rv
        for x in val:
            rv.append(unwrap(x, cache))
    elif isinstance(val, set):
        rv = set()
        cache[id(val)] = rv
        for x in val:
            rv.add(unwrap(x, cache))
    elif isinstance(val, dict):
        rv = {}
        cache[id(val)] = rv
        for k, v in list(val.items()):
            rv[unwrap(k, cache)] = unwrap(v, cache)
    elif isinstance(val, omero.RCollection):
        if val.val is None:
            rv = None
            cache[id(val)] = None
        else:
            rv = []
            cache[id(val)] = rv
            for x in val.val:
                rv.append(unwrap(x, cache))
    elif isinstance(val, omero.RMap):
        if val.val is None:
            rv = None
            cache[id(val)] = None
        else:
            rv = {}
            cache[id(val)] = rv
            for k, v in list(val.val.items()):
                rv[unwrap(k, cache)] = unwrap(v, cache)
    elif isinstance(val, omero.RType):  # Non-recursive
        rv = val.val
        cache[id(val)] = rv
    else:
        rv = val

    return rv


# Static factory methods (primitives)
# =========================================================================


def rbool(val):
    """
    Returns the argument itself if None or an instance of RBool.
    Otherwise, checks the value for"trueness" and returns either
    rtrue or rfalse.
    """
    if val is None or isinstance(val, omero.RBool):
        return val
    elif val:
        return rtrue
    else:
        return rfalse


def rdouble(val):
    """
    Returns the argument itself if None or an instance of RDouble.
    Otherwise, assigns a coerced float to the value of a new RDouble.
    """
    if val is None or isinstance(val, omero.RDouble):
        return val
    return RDoubleI(val)


def rfloat(val):
    """
    Returns the argument itself if None or an instance of RFloat.
    Otherwise, assigns a coerced float to the value of a new RFloat.
    """
    if val is None or isinstance(val, omero.RFloat):
        return val
    return RFloatI(val)


def rint(val):
    """
    Returns the argument itself if None or an instance of RInt.
    If the argument is 0, rint0 is returned.
    Otherwise, assigns a coerced int to the value of a new RInt
    """
    if val is None or isinstance(val, omero.RInt):
        return val
    elif val == 0:
        return rint0
    return RIntI(val)


def rlong(val):
    """
    Returns the argument itself if None or an instance of RLong.
    If the argument is 0, rlong 0 is returned.
    Otherwise, assigns a coerced int to the value of a new RLong
    """
    if val is None or isinstance(val, omero.RLong):
        return val
    elif val == 0:
        return rlong0
    return RLongI(val)


def rtime(val):
    """
    Returns the argument itself if None or an instance of RTime.
    Otherwise, assigns a coerced long to the value of a new RTime
    """
    if val is None or isinstance(val, omero.RTime):
        return val
    return RTimeI(val)

# Static factory methods (objects)
# =========================================================================


def rinternal(val):
    """
    If argument is None, returns rnullinternal.
    If an RInternal, returns the argument itself.
    Otherwise creates a new RInternal.
    """
    if val is None:
        return rnullinternal
    elif isinstance(val, omero.RInternal):
        return val
    elif isinstance(val, omero.Internal):
        return RInternalI(val)
    else:
        raise ValueError("Not Internal type: %s" % type(val))


def robject(val):
    """
    If argument is None, returns rnullobject.
    If an RObject, returns the argument itself.
    Otherwise creates a new RObject
    """
    if val is None:
        return rnullobject
    elif isinstance(val, omero.RObject):
        return val
    elif isinstance(val, omero.model.IObject):
        return RObjectI(val)
    else:
        raise ValueError("Not IObject type: %s" % type(val))


def rclass(val):
    """
    If argument is None or "", returns emptyclass.
    If an RClass, returns the argument itself.
    Otherwise creates a new RClass
    """
    if val is None:
        return remptyclass
    elif isinstance(val, omero.RClass):
        return val
    elif isinstance(val, basestring):
        if len(val) == 0:
            return remptyclass
        else:
            return RClassI(val)
    raise ValueError("Not string type: %s" % type(val))


def rstring(val):
    """
    If argument is None or "", returns emptystring.
    If an RString, returns the argument itself.
    Otherwise creates a new RString
    """
    if val is None:
        return remptystr
    elif isinstance(val, omero.RString):
        return val
    elif isinstance(val, basestring):
        if len(val) == 0:
            return remptystr
        else:
            if sys.version_info < (3, 0, 0):
                if isinstance(val, str):
                    val = val.encode("utf-8")
            return RStringI(val)
    else:
        return rstring(str(val))

# Static factory methods (collections)
# =========================================================================


def rarray(val=None, *args):
    return RArrayI(val, *args)


def rlist(val=None, *args):
    return RListI(val, *args)


def rset(val=None, *args):
    return RSetI(val, *args)


def rmap(val=None, **kwargs):
    return RMapI(val, **kwargs)

# Implementations (primitives)
# =========================================================================


class RBoolI(omero.RBool):

    def __init__(self, value):
        omero.RBool.__init__(self, value)

    def getValue(self, current=None):
        return self._val

    def compare(self, rhs, current=None):
        raise NotImplementedError("compare")

    def __eq__(self, obj):
        if obj is None:
            return False
        elif obj is self:
            return True
        elif not isinstance(obj, omero.RBool):
            return False
        return obj._val == self._val

    def __ne__(self, obj):
        return not self.__eq__(obj)

    def __hash__(self):
        if self._val:
            return hash(True)
        else:
            return hash(False)

    def __getattr__(self, attr):
        if attr == "val":
            return self.getValue()
        else:
            raise AttributeError(attr)

    def __setattr__(self, attr, value):
        if attr == "val":
            if "_val" in self.__dict__:
                raise omero.ClientError("Cannot write to val")
            else:
                self.__dict__["_val"] = value
        else:
            object.__setattr__(self, attr, value)

    def ice_postUnmarshal(self):
        """
        Provides additional initialization once all data loaded
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

    def ice_preMarshal(self):
        """
        Provides additional validation before data is sent
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused


class RDoubleI(omero.RDouble):

    def __init__(self, value):
        omero.RDouble.__init__(self, float(value))

    def getValue(self, current=None):
        return self._val

    def compare(self, rhs, current=None):
        raise NotImplementedError("compare")

    def __eq__(self, obj):
        if obj is None:
            return False
        elif obj is self:
            return True
        elif not isinstance(obj, omero.RDouble):
            return False
        return obj._val == self._val

    def __ne__(self, obj):
        return not self.__eq__(obj)

    def __hash__(self):
        return hash(self._val)

    def __getattr__(self, attr):
        if attr == "val":
            return self.getValue()
        else:
            raise AttributeError(attr)

    def __setattr__(self, attr, value):
        if attr == "val":
            if "_val" in self.__dict__:
                raise omero.ClientError("Cannot write to val")
            else:
                self.__dict__["_val"] = value
        else:
            object.__setattr__(self, attr, value)

    def ice_postUnmarshal(self):
        """
        Provides additional initialization once all data loaded
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

    def ice_preMarshal(self):
        """
        Provides additional validation before data is sent
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused


class RFloatI(omero.RFloat):

    def __init__(self, value):
        omero.RFloat.__init__(self, float(value))

    def getValue(self, current=None):
        return self._val

    def compare(self, rhs, current=None):
        raise NotImplementedError("compare")

    def __eq__(self, obj):
        if obj is None:
            return False
        elif obj is self:
            return True
        elif not isinstance(obj, omero.RFloat):
            return False
        return obj._val == self._val

    def __ne__(self, obj):
        return not self.__eq__(obj)

    def __hash__(self):
        return hash(self._val)

    def __getattr__(self, attr):
        if attr == "val":
            return self.getValue()
        else:
            raise AttributeError(attr)

    def __setattr__(self, attr, value):
        if attr == "val":
            if "_val" in self.__dict__:
                raise omero.ClientError("Cannot write to val")
            else:
                self.__dict__["_val"] = value
        else:
            object.__setattr__(self, attr, value)

    def ice_postUnmarshal(self):
        """
        Provides additional initialization once all data loaded
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

    def ice_preMarshal(self):
        """
        Provides additional validation before data is sent
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused


class RIntI(omero.RInt):

    def __init__(self, value):
        omero.RInt.__init__(self, int(value))

    def getValue(self, current=None):
        return self._val

    def compare(self, rhs, current=None):
        raise NotImplementedError("compare")

    def __eq__(self, obj):
        if obj is None:
            return False
        elif obj is self:
            return True
        elif not isinstance(obj, omero.RInt):
            return False
        return obj._val == self._val

    def __ne__(self, obj):
        return not self.__eq__(obj)

    def __hash__(self):
        return hash(self._val)

    def __getattr__(self, attr):
        if attr == "val":
            return self.getValue()
        else:
            raise AttributeError(attr)

    def __setattr__(self, attr, value):
        if attr == "val":
            if "_val" in self.__dict__:
                raise omero.ClientError("Cannot write to val")
            else:
                self.__dict__["_val"] = value
        else:
            object.__setattr__(self, attr, value)

    def ice_postUnmarshal(self):
        """
        Provides additional initialization once all data loaded
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

    def ice_preMarshal(self):
        """
        Provides additional validation before data is sent
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused


class RLongI(omero.RLong):

    def __init__(self, value):
        omero.RLong.__init__(self, long(value))

    def getValue(self, current=None):
        return self._val

    def compare(self, rhs, current=None):
        raise NotImplementedError("compare")

    def __eq__(self, obj):
        if obj is None:
            return False
        elif obj is self:
            return True
        elif not isinstance(obj, omero.RLong):
            return False
        return obj._val == self._val

    def __ne__(self, obj):
        return not self.__eq__(obj)

    def __hash__(self):
        return hash(self._val)

    def __getattr__(self, attr):
        if attr == "val":
            return self.getValue()
        else:
            raise AttributeError(attr)

    def __setattr__(self, attr, value):
        if attr == "val":
            if "_val" in self.__dict__:
                raise omero.ClientError("Cannot write to val")
            else:
                self.__dict__["_val"] = value
        else:
            object.__setattr__(self, attr, value)

    def ice_postUnmarshal(self):
        """
        Provides additional initialization once all data loaded
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

    def ice_preMarshal(self):
        """
        Provides additional validation before data is sent
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused


class RTimeI(omero.RTime):

    def __init__(self, value):
        omero.RTime.__init__(self, long(value))

    def getValue(self, current=None):
        return self._val

    def compare(self, rhs, current=None):
        raise NotImplementedError("compare")

    def __eq__(self, obj):
        if obj is None:
            return False
        elif obj is self:
            return True
        elif not isinstance(obj, omero.RTime):
            return False
        return obj._val == self._val

    def __ne__(self, obj):
        return not self.__eq__(obj)

    def __hash__(self):
        return hash(self._val)

    def __getattr__(self, attr):
        if attr == "val":
            return self.getValue()
        else:
            raise AttributeError(attr)

    def __setattr__(self, attr, value):
        if attr == "val":
            if "_val" in self.__dict__:
                raise omero.ClientError("Cannot write to val")
            else:
                self.__dict__["_val"] = value
        else:
            object.__setattr__(self, attr, value)

    def ice_postUnmarshal(self):
        """
        Provides additional initialization once all data loaded
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

    def ice_preMarshal(self):
        """
        Provides additional validation before data is sent
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

# Implementations (objects)
# =========================================================================


class RInternalI(omero.RInternal):

    def __init__(self, value):
        omero.RInternal.__init__(self, value)

    def getValue(self, current=None):
        return self._val

    def compare(self, rhs, current=None):
        raise NotImplementedError("compare")

    def __eq__(self, obj):
        if obj is None:
            return False
        elif obj is self:
            return True
        elif not isinstance(obj, omero.RInternal):
            return False
        return obj._val == self._val

    def __ne__(self, obj):
        return not self.__eq__(obj)

    def __hash__(self):
        return hash(self._val)

    def __getattr__(self, attr):
        if attr == "val":
            return self.getValue()
        else:
            raise AttributeError(attr)

    def __setattr__(self, attr, value):
        if attr == "val":
            if "_val" in self.__dict__:
                raise omero.ClientError("Cannot write to val")
            else:
                self.__dict__["_val"] = value
        else:
            object.__setattr__(self, attr, value)

    def ice_postUnmarshal(self):
        """
        Provides additional initialization once all data loaded
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

    def ice_preMarshal(self):
        """
        Provides additional validation before data is sent
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused


class RObjectI(omero.RObject):

    def __init__(self, value):
        omero.RObject.__init__(self, value)

    def getValue(self, current=None):
        return self._val

    def compare(self, rhs, current=None):
        raise NotImplementedError("compare")

    def __eq__(self, obj):
        if obj is None:
            return False
        elif obj is self:
            return True
        elif not isinstance(obj, omero.RObject):
            return False
        return obj._val == self._val

    def __ne__(self, obj):
        return not self.__eq__(obj)

    def __hash__(self):
        return hash(self._val)

    def __getattr__(self, attr):
        if attr == "val":
            return self.getValue()
        else:
            raise AttributeError(attr)

    def __setattr__(self, attr, value):
        if attr == "val":
            if "_val" in self.__dict__:
                raise omero.ClientError("Cannot write to val")
            else:
                self.__dict__["_val"] = value
        else:
            object.__setattr__(self, attr, value)

    def ice_postUnmarshal(self):
        """
        Provides additional initialization once all data loaded
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

    def ice_preMarshal(self):
        """
        Provides additional validation before data is sent
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused


class RStringI(omero.RString):

    def __init__(self, value):
        omero.RString.__init__(self, value)

    def getValue(self, current=None):
        return self._val

    def compare(self, rhs, current=None):
        raise NotImplementedError("compare")

    def __eq__(self, obj):
        if obj is None:
            return False
        elif obj is self:
            return True
        elif not isinstance(obj, omero.RString):
            return False
        return obj._val == self._val

    def __ne__(self, obj):
        return not self.__eq__(obj)

    def __hash__(self):
        return hash(self._val)

    def __getattr__(self, attr):
        if attr == "val":
            return self.getValue()
        else:
            raise AttributeError(attr)

    def __setattr__(self, attr, value):
        if attr == "val":
            if "_val" in self.__dict__:
                raise omero.ClientError("Cannot write to val")
            else:
                self.__dict__["_val"] = value
        else:
            object.__setattr__(self, attr, value)

    def ice_postUnmarshal(self):
        """
        Provides additional initialization once all data loaded
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

    def ice_preMarshal(self):
        """
        Provides additional validation before data is sent
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused


class RClassI(omero.RClass):

    def __init__(self, value):
        omero.RClass.__init__(self, value)

    def getValue(self, current=None):
        return self._val

    def compare(self, rhs, current=None):
        raise NotImplementedError("compare")

    def __eq__(self, obj):
        if obj is None:
            return False
        elif obj is self:
            return True
        elif not isinstance(obj, omero.RClass):
            return False
        return obj._val == self._val

    def __ne__(self, obj):
        return not self.__eq__(obj)

    def __hash__(self):
        return hash(self._val)

    def __getattr__(self, attr):
        if attr == "val":
            return self.getValue()
        else:
            raise AttributeError(attr)

    def __setattr__(self, attr, value):
        if attr == "val":
            if "_val" in self.__dict__:
                raise omero.ClientError("Cannot write to val")
            else:
                self.__dict__["_val"] = value
        else:
            object.__setattr__(self, attr, value)

    def ice_postUnmarshal(self):
        """
        Provides additional initialization once all data loaded
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

    def ice_preMarshal(self):
        """
        Provides additional validation before data is sent
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

# Implementations (collections)
# =========================================================================


class RArrayI(omero.RArray):
    """
    Guaranteed to never contain an empty list.
    """

    def __init__(self, arg=None, *args):
        if arg is None:
            self._val = []
        elif not hasattr(arg, "__iter__"):
            self._val = [arg]
        else:
            self._val = list(arg)
        self._val.extend(args)
        for v in self._val:
            if not isinstance(v, omero.RType):
                raise ValueError("Item of wrong type: %s" % type(v))

    def compare(self, rhs, current=None):
        raise NotImplementedError("compare")

    def getValue(self, current=None):
        return self._val

    def get(self, index, current=None):
        return self._val[index]

    def size(self, current=None):
        return len(self._val)

    def add(self, value, current=None):
        self._val.append(value)

    def addAll(self, values, current=None):
        self.val.append(values)

    def __eq__(self, obj):
        if obj is None:
            return False
        elif self is obj:
            return True
        elif not isinstance(obj, omero.RArray):
            return False
        return self._val == obj._val

    def __ne__(self, obj):
        return not self.__eq__(obj)

    def __hash__(self, obj):
        """
        Not allowed. Hashing a list is not supported.
        """
        return hash(self._val)  # Throws an exception

    def __getattr__(self, attr):
        if attr == "val":
            return self.getValue()
        else:
            raise AttributeError(attr)

    def __setattr__(self, attr, value):
        if attr == "val":
            if "_val" in self.__dict__:
                raise omero.ClientError("Cannot write to val")
            else:
                self.__dict__["_val"] = value
        else:
            object.__setattr__(self, attr, value)

    def ice_postUnmarshal(self):
        """
        Provides additional initialization once all data loaded
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

    def ice_preMarshal(self):
        """
        Provides additional validation before data is sent
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused


class RListI(omero.RList):
    """
    Guaranteed to never contain an empty list.
    """

    def __init__(self, arg=None, *args):
        if arg is None:
            self._val = []
        elif not hasattr(arg, "__iter__"):
            self._val = [arg]
        else:
            self._val = list(arg)
        self._val.extend(args)
        for v in self._val:
            if not isinstance(v, omero.RType):
                raise ValueError("Item of wrong type: %s" % type(v))

    def compare(self, rhs, current=None):
        raise NotImplementedError("compare")

    def getValue(self, current=None):
        return self._val

    def get(self, index, current=None):
        return self._val[index]

    def size(self, current=None):
        return len(self._val)

    def add(self, value, current=None):
        self._val.append(value)

    def addAll(self, values, current=None):
        self.val.append(values)

    def __eq__(self, obj):
        if obj is None:
            return False
        elif self is obj:
            return True
        elif not isinstance(obj, omero.RList):
            return False
        return self._val == obj._val

    def __ne__(self, obj):
        return not self.__eq__(obj)

    def __hash__(self, obj):
        """
        Not allowed. Hashing a list is not supported.
        """
        return hash(self._val)  # Throws an exception

    def __getattr__(self, attr):
        if attr == "val":
            return self.getValue()
        else:
            raise AttributeError(attr)

    def __setattr__(self, attr, value):
        if attr == "val":
            if "_val" in self.__dict__:
                raise omero.ClientError("Cannot write to val")
            else:
                self.__dict__["_val"] = value
        else:
            object.__setattr__(self, attr, value)

    def ice_postUnmarshal(self):
        """
        Provides additional initialization once all data loaded
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

    def ice_preMarshal(self):
        """
        Provides additional validation before data is sent
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused


class RSetI(omero.RSet):
    """
    Guaranteed to never contain an empty list.
    """

    def __init__(self, arg=None, *args):
        if arg is None:
            self._val = []
        elif not hasattr(arg, "__iter__"):
            self._val = [arg]
        else:
            self._val = list(arg)
        self._val.extend(args)
        for v in self._val:
            if not isinstance(v, omero.RType):
                raise ValueError("Item of wrong type: %s" % type(v))

    def compare(self, rhs, current=None):
        raise NotImplementedError("compare")

    def getValue(self, current=None):
        return self._val

    def get(self, index, current=None):
        return self._val[index]

    def size(self, current=None):
        return len(self._val)

    def add(self, value, current=None):
        self._val.append(value)

    def addAll(self, values, current=None):
        self.val.append(values)

    def __eq__(self, obj):
        if obj is None:
            return False
        elif self is obj:
            return True
        elif not isinstance(obj, omero.RSet):
            return False
        return self._val == obj._val

    def __ne__(self, obj):
        return not self.__eq__(obj)

    def __hash__(self, obj):
        """
        Not allowed. Hashing a list is not supported.
        """
        return hash(self._val)  # Throws an exception

    def __getattr__(self, attr):
        if attr == "val":
            return self.getValue()
        else:
            raise AttributeError(attr)

    def __setattr__(self, attr, value):
        if attr == "val":
            if "_val" in self.__dict__:
                raise omero.ClientError("Cannot write to val")
            else:
                self.__dict__["_val"] = value
        else:
            object.__setattr__(self, attr, value)

    def ice_postUnmarshal(self):
        """
        Provides additional initialization once all data loaded
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

    def ice_preMarshal(self):
        """
        Provides additional validation before data is sent
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused


class RMapI(omero.RMap):

    def __init__(self, arg=None, **kwargs):
        if arg is None:
            self._val = {}
        else:
            self._val = dict(arg)  # May throw an exception
        self._val.update(kwargs)
        self._validate()

    def _validate(self):
        for k, v in list(self._val.items()):
            if not isinstance(k, basestring):
                raise ValueError("Key of wrong type: %s" % type(k))
            if v is not None and not isinstance(v, omero.RType):
                raise ValueError("Value of wrong type: %s" % type(v))

    def compare(self, rhs, current=None):
        raise NotImplementedError("compare")

    def getValue(self, current=None):
        return self._val

    def get(self, key, current=None):
        return self._val[key]

    def put(self, key, value, current=None):
        self._val[key] = value

    def size(self, current=None):
        return len(self._val)

    def __eq__(self, obj):
        if obj is None:
            return False
        elif self is obj:
            return True
        elif not isinstance(obj, omero.RMap):
            return False
        return self._val == obj._val

    def __ne__(self, obj):
        return not self.__eq__(obj)

    def __hash__(self, obj):
        """
        Not allowed. Hashing a list is not supported.
        """
        return hash(self._val)  # Throws an exception

    def __getattr__(self, attr):
        if attr == "val":
            return self.getValue()
        else:
            raise AttributeError(attr)

    def __setattr__(self, attr, value):
        if attr == "val":
            if "_val" in self.__dict__:
                raise omero.ClientError("Cannot write to val")
            else:
                self.__dict__["_val"] = value
        else:
            object.__setattr__(self, attr, value)

    def ice_postUnmarshal(self):
        """
        Provides additional initialization once all data loaded
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

    def ice_preMarshal(self):
        """
        Provides additional validation before data is sent
        Required due to __getattr__ implementation.
        """
        pass  # Currently unused

# Helpers
# ========================================================================

# Conversion classes are for omero.model <--> ome.model only (no python)


class ObjectFactory(Ice.ObjectFactory):

    def __init__(self, cls, f):
        try:
            self.id = cls.ice_staticId()
        except Exception:
            pass
        self.f = f

    def create(self, string):
        return self.f()

    def destroy(self):
        pass

    def register(self, ic):
        ic.addObjectFactory(self, self.id)


# Shared state (flyweight)
# =========================================================================

rtrue = RBoolI(True)

rfalse = RBoolI(False)

rlong0 = RLongI(0)

rint0 = RIntI(0)

remptystr = RStringI("")

remptyclass = RClassI("")

rnullinternal = RInternalI(None)

rnullobject = RObjectI(None)

# Object factories
# =========================================================================

ObjectFactories = {
    RBoolI: ObjectFactory(RBoolI, lambda: RBoolI(False)),
    RDoubleI: ObjectFactory(RDoubleI, lambda: RDoubleI(0.0)),
    RFloatI: ObjectFactory(RFloatI, lambda: RFloatI(0.0)),
    RIntI: ObjectFactory(RIntI, lambda: RIntI(0)),
    RLongI: ObjectFactory(RLongI, lambda: RLongI(0)),
    RTimeI: ObjectFactory(RTimeI, lambda: RTimeI(0)),
    RClassI: ObjectFactory(RClassI, lambda: RClassI("")),
    RStringI: ObjectFactory(RStringI, lambda: RStringI("")),
    RInternalI: ObjectFactory(RInternalI, lambda: RInternalI(None)),
    RObjectI: ObjectFactory(RObjectI, lambda: RObjectI(None)),
    RArrayI: ObjectFactory(RArrayI, lambda: RArrayI()),
    RListI: ObjectFactory(RListI, lambda: RListI()),
    RSetI: ObjectFactory(RSetI, lambda: RSetI()),
    RMapI: ObjectFactory(RMapI, lambda: RMapI())
    }



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright 2009 Glencoe Software, Inc.  All Rights Reserved.
# Use is subject to license terms supplied in LICENSE.txt
#

"""
OMERO Concurrency Utilities
"""
import atexit
import logging
import threading
import omero.util
import logging.handlers

try:
    from threading import _Event
    from threading import _Timer
except ImportError:
    # Python3
    from threading import Event as _Event
    from threading import Timer as _Timer


def get_event(name="Unknown"):
    """
    Returns a threading.Event instance which is registered to be
    "set" (Event.set()) on system exit.
    """
    event = AtExitEvent(name=name)
    atexit.register(event.setAtExit)
    return event


class AtExitEvent(_Event):

    """
    threading.Event extension which provides an additional method
    setAtExit() which sets "atexit" to true.

    This class was introduced in 4.2.1 to work around issue #3260
    in which logging from background threads produced error
    messages.
    """

    def __init__(self, verbose=None, name="Unknown"):
        try:
            super(AtExitEvent, self).__init__(verbose)
        except TypeError:
            # in Python 3 there is no verbose argument
            super(AtExitEvent, self).__init__()
        self.__name = name
        self.__atexit = False

    name = property(lambda self: self.__name)
    atexit = property(lambda self: self.__atexit)

    def setAtExit(self):
        self.__atexit = True
        super(AtExitEvent, self).set()

    def __repr__(self):
        return "%s (%s)" % (super(AtExitEvent, self).__repr__(), self.__name)


class Timer(_Timer):

    """Based on threading._Thread but allows for resetting the Timer.

    t = Timer(30.0, f, args=[], kwargs={})
    t.start()
    t.cancel() # stop the timer's action if it's still waiting

    # or

    t.reset()

    After excecution, the status of the run can be checked via the
    "completed" and the "exception" Event instances.
    """

    def __init__(self, interval, function, args=None, kwargs=None):
        if args is None:
            args = []
        if kwargs is None:
            kwargs = {}
        _Timer.__init__(self, interval, function, args, kwargs)
        self.log = logging.getLogger(omero.util.make_logname(self))
        self.completed = threading.Event()
        self.exception = threading.Event()
        self._reset = threading.Event()

    def reset(self):
        self.log.debug("Reset called")
        self._reset.set()  # Set first, so that the loop will continue
        self.finished.set()  # Forces waiting thread to fall through

    def run(self):
        while True:
            self.finished.wait(self.interval)
            if self._reset.isSet():
                self.finished.clear()
                self._reset.clear()
                self.log.debug("Resetting")
                continue
            if not self.finished.isSet():
                try:
                    self.log.debug("Executing")
                    self.function(*self.args, **self.kwargs)
                    self.completed.set()
                    self.finished.set()
                except:
                    self.exception.set()
                    self.finished.set()
                    raise
            break



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright 2007 Glencoe Software, Inc. All rights reserved.
# Use is subject to license terms supplied in LICENSE.txt

"""
submit plugin

Plugin read by omero.cli.Cli during initialization. The method(s)
defined here will be added to the Cli class for later use.
"""
from __future__ import print_function

from builtins import str
from omero.cli import BaseControl, CLI
import sys

prompt = "omero submit [%s]> "


class Save(Exception):
    pass


class Cancel(Exception):
    pass


class SubmitCLI(CLI):

    def __init__(self):
        CLI.__init__(self)
        self.queue = []
        self.prompt = prompt % str(0)

    def postcmd(self, stop, line):
        self.queue.append(line)
        self.prompt = prompt % str(len(self.queue))
        return CLI.postcmd(self, stop, line)

    def do_save(self, arg):
        raise Save()

    def do_cancel(self, arg):
        raise Cancel()

    def post_process(self):
        print("Uploading")
        print(self.queue)

HELP = """When run without arguments, submit shell is opened
which takes commands without executing them. On save,
the file is trasferred to the server, and executed."""


class SubmitControl(BaseControl):

    def _configure(self, parser):
        parser.add_argument("arg", nargs="*", help="single command with args")
        parser.set_defaults(func=self.__call__)

    def __call__(self, args):
        submit = SubmitCLI()
        arg = args.arg
        if arg and len(arg) > 0:
            submit.invoke(arg)
            submit.post_process()
        else:
            try:
                submit.invokeloop()
            except Save:
                submit.execute()
            except Cancel:
                l = len(submit.queue)
                if l > 0:
                    print(l, " items queued. Really cancel? [Yn]")

try:
    # register("submit", SubmitControl, HELP)
    pass
except NameError:
    if __name__ == "__main__":
        cli = CLI()
        cli.register("submit", SubmitControl, HELP)
        cli.invoke(sys.argv[1:])



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Instrument.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'Microscope' not in _M_omero.model.__dict__:
    _M_omero.model._t_Microscope = IcePy.declareClass('::omero::model::Microscope')
    _M_omero.model._t_MicroscopePrx = IcePy.declareProxy('::omero::model::Microscope')

if 'Detector' not in _M_omero.model.__dict__:
    _M_omero.model._t_Detector = IcePy.declareClass('::omero::model::Detector')
    _M_omero.model._t_DetectorPrx = IcePy.declareProxy('::omero::model::Detector')

if 'Objective' not in _M_omero.model.__dict__:
    _M_omero.model._t_Objective = IcePy.declareClass('::omero::model::Objective')
    _M_omero.model._t_ObjectivePrx = IcePy.declareProxy('::omero::model::Objective')

if 'LightSource' not in _M_omero.model.__dict__:
    _M_omero.model._t_LightSource = IcePy.declareClass('::omero::model::LightSource')
    _M_omero.model._t_LightSourcePrx = IcePy.declareProxy('::omero::model::LightSource')

if 'Filter' not in _M_omero.model.__dict__:
    _M_omero.model._t_Filter = IcePy.declareClass('::omero::model::Filter')
    _M_omero.model._t_FilterPrx = IcePy.declareProxy('::omero::model::Filter')

if 'Dichroic' not in _M_omero.model.__dict__:
    _M_omero.model._t_Dichroic = IcePy.declareClass('::omero::model::Dichroic')
    _M_omero.model._t_DichroicPrx = IcePy.declareProxy('::omero::model::Dichroic')

if 'FilterSet' not in _M_omero.model.__dict__:
    _M_omero.model._t_FilterSet = IcePy.declareClass('::omero::model::FilterSet')
    _M_omero.model._t_FilterSetPrx = IcePy.declareProxy('::omero::model::FilterSet')

if 'OTF' not in _M_omero.model.__dict__:
    _M_omero.model._t_OTF = IcePy.declareClass('::omero::model::OTF')
    _M_omero.model._t_OTFPrx = IcePy.declareProxy('::omero::model::OTF')

if 'InstrumentAnnotationLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_InstrumentAnnotationLink = IcePy.declareClass('::omero::model::InstrumentAnnotationLink')
    _M_omero.model._t_InstrumentAnnotationLinkPrx = IcePy.declareProxy('::omero::model::InstrumentAnnotationLink')

if 'Annotation' not in _M_omero.model.__dict__:
    _M_omero.model._t_Annotation = IcePy.declareClass('::omero::model::Annotation')
    _M_omero.model._t_AnnotationPrx = IcePy.declareProxy('::omero::model::Annotation')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if '_t_InstrumentDetectorSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_InstrumentDetectorSeq = IcePy.defineSequence('::omero::model::InstrumentDetectorSeq', (), _M_omero.model._t_Detector)

if '_t_InstrumentObjectiveSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_InstrumentObjectiveSeq = IcePy.defineSequence('::omero::model::InstrumentObjectiveSeq', (), _M_omero.model._t_Objective)

if '_t_InstrumentLightSourceSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_InstrumentLightSourceSeq = IcePy.defineSequence('::omero::model::InstrumentLightSourceSeq', (), _M_omero.model._t_LightSource)

if '_t_InstrumentFilterSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_InstrumentFilterSeq = IcePy.defineSequence('::omero::model::InstrumentFilterSeq', (), _M_omero.model._t_Filter)

if '_t_InstrumentDichroicSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_InstrumentDichroicSeq = IcePy.defineSequence('::omero::model::InstrumentDichroicSeq', (), _M_omero.model._t_Dichroic)

if '_t_InstrumentFilterSetSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_InstrumentFilterSetSeq = IcePy.defineSequence('::omero::model::InstrumentFilterSetSeq', (), _M_omero.model._t_FilterSet)

if '_t_InstrumentOtfSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_InstrumentOtfSeq = IcePy.defineSequence('::omero::model::InstrumentOtfSeq', (), _M_omero.model._t_OTF)

if '_t_InstrumentAnnotationLinksSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_InstrumentAnnotationLinksSeq = IcePy.defineSequence('::omero::model::InstrumentAnnotationLinksSeq', (), _M_omero.model._t_InstrumentAnnotationLink)

if '_t_InstrumentLinkedAnnotationSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_InstrumentLinkedAnnotationSeq = IcePy.defineSequence('::omero::model::InstrumentLinkedAnnotationSeq', (), _M_omero.model._t_Annotation)

if 'Instrument' not in _M_omero.model.__dict__:
    _M_omero.model.Instrument = Ice.createTempClass()
    class Instrument(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _microscope=None, _detectorSeq=None, _detectorLoaded=False, _objectiveSeq=None, _objectiveLoaded=False, _lightSourceSeq=None, _lightSourceLoaded=False, _filterSeq=None, _filterLoaded=False, _dichroicSeq=None, _dichroicLoaded=False, _filterSetSeq=None, _filterSetLoaded=False, _otfSeq=None, _otfLoaded=False, _annotationLinksSeq=None, _annotationLinksLoaded=False, _annotationLinksCountPerOwner=None):
            if Ice.getType(self) == _M_omero.model.Instrument:
                raise RuntimeError('omero.model.Instrument is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._microscope = _microscope
            self._detectorSeq = _detectorSeq
            self._detectorLoaded = _detectorLoaded
            self._objectiveSeq = _objectiveSeq
            self._objectiveLoaded = _objectiveLoaded
            self._lightSourceSeq = _lightSourceSeq
            self._lightSourceLoaded = _lightSourceLoaded
            self._filterSeq = _filterSeq
            self._filterLoaded = _filterLoaded
            self._dichroicSeq = _dichroicSeq
            self._dichroicLoaded = _dichroicLoaded
            self._filterSetSeq = _filterSetSeq
            self._filterSetLoaded = _filterSetLoaded
            self._otfSeq = _otfSeq
            self._otfLoaded = _otfLoaded
            self._annotationLinksSeq = _annotationLinksSeq
            self._annotationLinksLoaded = _annotationLinksLoaded
            self._annotationLinksCountPerOwner = _annotationLinksCountPerOwner

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::IObject', '::omero::model::Instrument')

        def ice_id(self, current=None):
            return '::omero::model::Instrument'

        def ice_staticId():
            return '::omero::model::Instrument'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def getMicroscope(self, current=None):
            pass

        def setMicroscope(self, theMicroscope, current=None):
            pass

        def unloadDetector(self, current=None):
            pass

        def sizeOfDetector(self, current=None):
            pass

        def copyDetector(self, current=None):
            pass

        def addDetector(self, target, current=None):
            pass

        def addAllDetectorSet(self, targets, current=None):
            pass

        def removeDetector(self, theTarget, current=None):
            pass

        def removeAllDetectorSet(self, targets, current=None):
            pass

        def clearDetector(self, current=None):
            pass

        def reloadDetector(self, toCopy, current=None):
            pass

        def unloadObjective(self, current=None):
            pass

        def sizeOfObjective(self, current=None):
            pass

        def copyObjective(self, current=None):
            pass

        def addObjective(self, target, current=None):
            pass

        def addAllObjectiveSet(self, targets, current=None):
            pass

        def removeObjective(self, theTarget, current=None):
            pass

        def removeAllObjectiveSet(self, targets, current=None):
            pass

        def clearObjective(self, current=None):
            pass

        def reloadObjective(self, toCopy, current=None):
            pass

        def unloadLightSource(self, current=None):
            pass

        def sizeOfLightSource(self, current=None):
            pass

        def copyLightSource(self, current=None):
            pass

        def addLightSource(self, target, current=None):
            pass

        def addAllLightSourceSet(self, targets, current=None):
            pass

        def removeLightSource(self, theTarget, current=None):
            pass

        def removeAllLightSourceSet(self, targets, current=None):
            pass

        def clearLightSource(self, current=None):
            pass

        def reloadLightSource(self, toCopy, current=None):
            pass

        def unloadFilter(self, current=None):
            pass

        def sizeOfFilter(self, current=None):
            pass

        def copyFilter(self, current=None):
            pass

        def addFilter(self, target, current=None):
            pass

        def addAllFilterSet(self, targets, current=None):
            pass

        def removeFilter(self, theTarget, current=None):
            pass

        def removeAllFilterSet(self, targets, current=None):
            pass

        def clearFilter(self, current=None):
            pass

        def reloadFilter(self, toCopy, current=None):
            pass

        def unloadDichroic(self, current=None):
            pass

        def sizeOfDichroic(self, current=None):
            pass

        def copyDichroic(self, current=None):
            pass

        def addDichroic(self, target, current=None):
            pass

        def addAllDichroicSet(self, targets, current=None):
            pass

        def removeDichroic(self, theTarget, current=None):
            pass

        def removeAllDichroicSet(self, targets, current=None):
            pass

        def clearDichroic(self, current=None):
            pass

        def reloadDichroic(self, toCopy, current=None):
            pass

        def unloadFilterSet(self, current=None):
            pass

        def sizeOfFilterSet(self, current=None):
            pass

        def copyFilterSet(self, current=None):
            pass

        def addFilterSet(self, target, current=None):
            pass

        def addAllFilterSetSet(self, targets, current=None):
            pass

        def removeFilterSet(self, theTarget, current=None):
            pass

        def removeAllFilterSetSet(self, targets, current=None):
            pass

        def clearFilterSet(self, current=None):
            pass

        def reloadFilterSet(self, toCopy, current=None):
            pass

        def unloadOtf(self, current=None):
            pass

        def sizeOfOtf(self, current=None):
            pass

        def copyOtf(self, current=None):
            pass

        def addOTF(self, target, current=None):
            pass

        def addAllOTFSet(self, targets, current=None):
            pass

        def removeOTF(self, theTarget, current=None):
            pass

        def removeAllOTFSet(self, targets, current=None):
            pass

        def clearOtf(self, current=None):
            pass

        def reloadOtf(self, toCopy, current=None):
            pass

        def unloadAnnotationLinks(self, current=None):
            pass

        def sizeOfAnnotationLinks(self, current=None):
            pass

        def copyAnnotationLinks(self, current=None):
            pass

        def addInstrumentAnnotationLink(self, target, current=None):
            pass

        def addAllInstrumentAnnotationLinkSet(self, targets, current=None):
            pass

        def removeInstrumentAnnotationLink(self, theTarget, current=None):
            pass

        def removeAllInstrumentAnnotationLinkSet(self, targets, current=None):
            pass

        def clearAnnotationLinks(self, current=None):
            pass

        def reloadAnnotationLinks(self, toCopy, current=None):
            pass

        def getAnnotationLinksCountPerOwner(self, current=None):
            pass

        def linkAnnotation(self, addition, current=None):
            pass

        def addInstrumentAnnotationLinkToBoth(self, link, bothSides, current=None):
            pass

        def findInstrumentAnnotationLink(self, removal, current=None):
            pass

        def unlinkAnnotation(self, removal, current=None):
            pass

        def removeInstrumentAnnotationLinkFromBoth(self, link, bothSides, current=None):
            pass

        def linkedAnnotationList(self, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_Instrument)

        __repr__ = __str__

    _M_omero.model.InstrumentPrx = Ice.createTempClass()
    class InstrumentPrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.Instrument._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.Instrument._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.Instrument._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.Instrument._op_setVersion.end(self, _r)

        def getMicroscope(self, _ctx=None):
            return _M_omero.model.Instrument._op_getMicroscope.invoke(self, ((), _ctx))

        def begin_getMicroscope(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_getMicroscope.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getMicroscope(self, _r):
            return _M_omero.model.Instrument._op_getMicroscope.end(self, _r)

        def setMicroscope(self, theMicroscope, _ctx=None):
            return _M_omero.model.Instrument._op_setMicroscope.invoke(self, ((theMicroscope, ), _ctx))

        def begin_setMicroscope(self, theMicroscope, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_setMicroscope.begin(self, ((theMicroscope, ), _response, _ex, _sent, _ctx))

        def end_setMicroscope(self, _r):
            return _M_omero.model.Instrument._op_setMicroscope.end(self, _r)

        def unloadDetector(self, _ctx=None):
            return _M_omero.model.Instrument._op_unloadDetector.invoke(self, ((), _ctx))

        def begin_unloadDetector(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_unloadDetector.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadDetector(self, _r):
            return _M_omero.model.Instrument._op_unloadDetector.end(self, _r)

        def sizeOfDetector(self, _ctx=None):
            return _M_omero.model.Instrument._op_sizeOfDetector.invoke(self, ((), _ctx))

        def begin_sizeOfDetector(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_sizeOfDetector.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfDetector(self, _r):
            return _M_omero.model.Instrument._op_sizeOfDetector.end(self, _r)

        def copyDetector(self, _ctx=None):
            return _M_omero.model.Instrument._op_copyDetector.invoke(self, ((), _ctx))

        def begin_copyDetector(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_copyDetector.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyDetector(self, _r):
            return _M_omero.model.Instrument._op_copyDetector.end(self, _r)

        def addDetector(self, target, _ctx=None):
            return _M_omero.model.Instrument._op_addDetector.invoke(self, ((target, ), _ctx))

        def begin_addDetector(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addDetector.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addDetector(self, _r):
            return _M_omero.model.Instrument._op_addDetector.end(self, _r)

        def addAllDetectorSet(self, targets, _ctx=None):
            return _M_omero.model.Instrument._op_addAllDetectorSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllDetectorSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addAllDetectorSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllDetectorSet(self, _r):
            return _M_omero.model.Instrument._op_addAllDetectorSet.end(self, _r)

        def removeDetector(self, theTarget, _ctx=None):
            return _M_omero.model.Instrument._op_removeDetector.invoke(self, ((theTarget, ), _ctx))

        def begin_removeDetector(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeDetector.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeDetector(self, _r):
            return _M_omero.model.Instrument._op_removeDetector.end(self, _r)

        def removeAllDetectorSet(self, targets, _ctx=None):
            return _M_omero.model.Instrument._op_removeAllDetectorSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllDetectorSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeAllDetectorSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllDetectorSet(self, _r):
            return _M_omero.model.Instrument._op_removeAllDetectorSet.end(self, _r)

        def clearDetector(self, _ctx=None):
            return _M_omero.model.Instrument._op_clearDetector.invoke(self, ((), _ctx))

        def begin_clearDetector(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_clearDetector.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearDetector(self, _r):
            return _M_omero.model.Instrument._op_clearDetector.end(self, _r)

        def reloadDetector(self, toCopy, _ctx=None):
            return _M_omero.model.Instrument._op_reloadDetector.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadDetector(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_reloadDetector.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadDetector(self, _r):
            return _M_omero.model.Instrument._op_reloadDetector.end(self, _r)

        def unloadObjective(self, _ctx=None):
            return _M_omero.model.Instrument._op_unloadObjective.invoke(self, ((), _ctx))

        def begin_unloadObjective(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_unloadObjective.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadObjective(self, _r):
            return _M_omero.model.Instrument._op_unloadObjective.end(self, _r)

        def sizeOfObjective(self, _ctx=None):
            return _M_omero.model.Instrument._op_sizeOfObjective.invoke(self, ((), _ctx))

        def begin_sizeOfObjective(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_sizeOfObjective.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfObjective(self, _r):
            return _M_omero.model.Instrument._op_sizeOfObjective.end(self, _r)

        def copyObjective(self, _ctx=None):
            return _M_omero.model.Instrument._op_copyObjective.invoke(self, ((), _ctx))

        def begin_copyObjective(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_copyObjective.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyObjective(self, _r):
            return _M_omero.model.Instrument._op_copyObjective.end(self, _r)

        def addObjective(self, target, _ctx=None):
            return _M_omero.model.Instrument._op_addObjective.invoke(self, ((target, ), _ctx))

        def begin_addObjective(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addObjective.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addObjective(self, _r):
            return _M_omero.model.Instrument._op_addObjective.end(self, _r)

        def addAllObjectiveSet(self, targets, _ctx=None):
            return _M_omero.model.Instrument._op_addAllObjectiveSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllObjectiveSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addAllObjectiveSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllObjectiveSet(self, _r):
            return _M_omero.model.Instrument._op_addAllObjectiveSet.end(self, _r)

        def removeObjective(self, theTarget, _ctx=None):
            return _M_omero.model.Instrument._op_removeObjective.invoke(self, ((theTarget, ), _ctx))

        def begin_removeObjective(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeObjective.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeObjective(self, _r):
            return _M_omero.model.Instrument._op_removeObjective.end(self, _r)

        def removeAllObjectiveSet(self, targets, _ctx=None):
            return _M_omero.model.Instrument._op_removeAllObjectiveSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllObjectiveSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeAllObjectiveSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllObjectiveSet(self, _r):
            return _M_omero.model.Instrument._op_removeAllObjectiveSet.end(self, _r)

        def clearObjective(self, _ctx=None):
            return _M_omero.model.Instrument._op_clearObjective.invoke(self, ((), _ctx))

        def begin_clearObjective(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_clearObjective.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearObjective(self, _r):
            return _M_omero.model.Instrument._op_clearObjective.end(self, _r)

        def reloadObjective(self, toCopy, _ctx=None):
            return _M_omero.model.Instrument._op_reloadObjective.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadObjective(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_reloadObjective.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadObjective(self, _r):
            return _M_omero.model.Instrument._op_reloadObjective.end(self, _r)

        def unloadLightSource(self, _ctx=None):
            return _M_omero.model.Instrument._op_unloadLightSource.invoke(self, ((), _ctx))

        def begin_unloadLightSource(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_unloadLightSource.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadLightSource(self, _r):
            return _M_omero.model.Instrument._op_unloadLightSource.end(self, _r)

        def sizeOfLightSource(self, _ctx=None):
            return _M_omero.model.Instrument._op_sizeOfLightSource.invoke(self, ((), _ctx))

        def begin_sizeOfLightSource(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_sizeOfLightSource.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfLightSource(self, _r):
            return _M_omero.model.Instrument._op_sizeOfLightSource.end(self, _r)

        def copyLightSource(self, _ctx=None):
            return _M_omero.model.Instrument._op_copyLightSource.invoke(self, ((), _ctx))

        def begin_copyLightSource(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_copyLightSource.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyLightSource(self, _r):
            return _M_omero.model.Instrument._op_copyLightSource.end(self, _r)

        def addLightSource(self, target, _ctx=None):
            return _M_omero.model.Instrument._op_addLightSource.invoke(self, ((target, ), _ctx))

        def begin_addLightSource(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addLightSource.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addLightSource(self, _r):
            return _M_omero.model.Instrument._op_addLightSource.end(self, _r)

        def addAllLightSourceSet(self, targets, _ctx=None):
            return _M_omero.model.Instrument._op_addAllLightSourceSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllLightSourceSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addAllLightSourceSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllLightSourceSet(self, _r):
            return _M_omero.model.Instrument._op_addAllLightSourceSet.end(self, _r)

        def removeLightSource(self, theTarget, _ctx=None):
            return _M_omero.model.Instrument._op_removeLightSource.invoke(self, ((theTarget, ), _ctx))

        def begin_removeLightSource(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeLightSource.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeLightSource(self, _r):
            return _M_omero.model.Instrument._op_removeLightSource.end(self, _r)

        def removeAllLightSourceSet(self, targets, _ctx=None):
            return _M_omero.model.Instrument._op_removeAllLightSourceSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllLightSourceSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeAllLightSourceSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllLightSourceSet(self, _r):
            return _M_omero.model.Instrument._op_removeAllLightSourceSet.end(self, _r)

        def clearLightSource(self, _ctx=None):
            return _M_omero.model.Instrument._op_clearLightSource.invoke(self, ((), _ctx))

        def begin_clearLightSource(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_clearLightSource.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearLightSource(self, _r):
            return _M_omero.model.Instrument._op_clearLightSource.end(self, _r)

        def reloadLightSource(self, toCopy, _ctx=None):
            return _M_omero.model.Instrument._op_reloadLightSource.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadLightSource(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_reloadLightSource.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadLightSource(self, _r):
            return _M_omero.model.Instrument._op_reloadLightSource.end(self, _r)

        def unloadFilter(self, _ctx=None):
            return _M_omero.model.Instrument._op_unloadFilter.invoke(self, ((), _ctx))

        def begin_unloadFilter(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_unloadFilter.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadFilter(self, _r):
            return _M_omero.model.Instrument._op_unloadFilter.end(self, _r)

        def sizeOfFilter(self, _ctx=None):
            return _M_omero.model.Instrument._op_sizeOfFilter.invoke(self, ((), _ctx))

        def begin_sizeOfFilter(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_sizeOfFilter.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfFilter(self, _r):
            return _M_omero.model.Instrument._op_sizeOfFilter.end(self, _r)

        def copyFilter(self, _ctx=None):
            return _M_omero.model.Instrument._op_copyFilter.invoke(self, ((), _ctx))

        def begin_copyFilter(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_copyFilter.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyFilter(self, _r):
            return _M_omero.model.Instrument._op_copyFilter.end(self, _r)

        def addFilter(self, target, _ctx=None):
            return _M_omero.model.Instrument._op_addFilter.invoke(self, ((target, ), _ctx))

        def begin_addFilter(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addFilter.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addFilter(self, _r):
            return _M_omero.model.Instrument._op_addFilter.end(self, _r)

        def addAllFilterSet(self, targets, _ctx=None):
            return _M_omero.model.Instrument._op_addAllFilterSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllFilterSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addAllFilterSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllFilterSet(self, _r):
            return _M_omero.model.Instrument._op_addAllFilterSet.end(self, _r)

        def removeFilter(self, theTarget, _ctx=None):
            return _M_omero.model.Instrument._op_removeFilter.invoke(self, ((theTarget, ), _ctx))

        def begin_removeFilter(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeFilter.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeFilter(self, _r):
            return _M_omero.model.Instrument._op_removeFilter.end(self, _r)

        def removeAllFilterSet(self, targets, _ctx=None):
            return _M_omero.model.Instrument._op_removeAllFilterSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllFilterSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeAllFilterSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllFilterSet(self, _r):
            return _M_omero.model.Instrument._op_removeAllFilterSet.end(self, _r)

        def clearFilter(self, _ctx=None):
            return _M_omero.model.Instrument._op_clearFilter.invoke(self, ((), _ctx))

        def begin_clearFilter(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_clearFilter.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearFilter(self, _r):
            return _M_omero.model.Instrument._op_clearFilter.end(self, _r)

        def reloadFilter(self, toCopy, _ctx=None):
            return _M_omero.model.Instrument._op_reloadFilter.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadFilter(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_reloadFilter.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadFilter(self, _r):
            return _M_omero.model.Instrument._op_reloadFilter.end(self, _r)

        def unloadDichroic(self, _ctx=None):
            return _M_omero.model.Instrument._op_unloadDichroic.invoke(self, ((), _ctx))

        def begin_unloadDichroic(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_unloadDichroic.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadDichroic(self, _r):
            return _M_omero.model.Instrument._op_unloadDichroic.end(self, _r)

        def sizeOfDichroic(self, _ctx=None):
            return _M_omero.model.Instrument._op_sizeOfDichroic.invoke(self, ((), _ctx))

        def begin_sizeOfDichroic(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_sizeOfDichroic.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfDichroic(self, _r):
            return _M_omero.model.Instrument._op_sizeOfDichroic.end(self, _r)

        def copyDichroic(self, _ctx=None):
            return _M_omero.model.Instrument._op_copyDichroic.invoke(self, ((), _ctx))

        def begin_copyDichroic(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_copyDichroic.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyDichroic(self, _r):
            return _M_omero.model.Instrument._op_copyDichroic.end(self, _r)

        def addDichroic(self, target, _ctx=None):
            return _M_omero.model.Instrument._op_addDichroic.invoke(self, ((target, ), _ctx))

        def begin_addDichroic(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addDichroic.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addDichroic(self, _r):
            return _M_omero.model.Instrument._op_addDichroic.end(self, _r)

        def addAllDichroicSet(self, targets, _ctx=None):
            return _M_omero.model.Instrument._op_addAllDichroicSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllDichroicSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addAllDichroicSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllDichroicSet(self, _r):
            return _M_omero.model.Instrument._op_addAllDichroicSet.end(self, _r)

        def removeDichroic(self, theTarget, _ctx=None):
            return _M_omero.model.Instrument._op_removeDichroic.invoke(self, ((theTarget, ), _ctx))

        def begin_removeDichroic(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeDichroic.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeDichroic(self, _r):
            return _M_omero.model.Instrument._op_removeDichroic.end(self, _r)

        def removeAllDichroicSet(self, targets, _ctx=None):
            return _M_omero.model.Instrument._op_removeAllDichroicSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllDichroicSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeAllDichroicSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllDichroicSet(self, _r):
            return _M_omero.model.Instrument._op_removeAllDichroicSet.end(self, _r)

        def clearDichroic(self, _ctx=None):
            return _M_omero.model.Instrument._op_clearDichroic.invoke(self, ((), _ctx))

        def begin_clearDichroic(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_clearDichroic.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearDichroic(self, _r):
            return _M_omero.model.Instrument._op_clearDichroic.end(self, _r)

        def reloadDichroic(self, toCopy, _ctx=None):
            return _M_omero.model.Instrument._op_reloadDichroic.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadDichroic(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_reloadDichroic.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadDichroic(self, _r):
            return _M_omero.model.Instrument._op_reloadDichroic.end(self, _r)

        def unloadFilterSet(self, _ctx=None):
            return _M_omero.model.Instrument._op_unloadFilterSet.invoke(self, ((), _ctx))

        def begin_unloadFilterSet(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_unloadFilterSet.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadFilterSet(self, _r):
            return _M_omero.model.Instrument._op_unloadFilterSet.end(self, _r)

        def sizeOfFilterSet(self, _ctx=None):
            return _M_omero.model.Instrument._op_sizeOfFilterSet.invoke(self, ((), _ctx))

        def begin_sizeOfFilterSet(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_sizeOfFilterSet.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfFilterSet(self, _r):
            return _M_omero.model.Instrument._op_sizeOfFilterSet.end(self, _r)

        def copyFilterSet(self, _ctx=None):
            return _M_omero.model.Instrument._op_copyFilterSet.invoke(self, ((), _ctx))

        def begin_copyFilterSet(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_copyFilterSet.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyFilterSet(self, _r):
            return _M_omero.model.Instrument._op_copyFilterSet.end(self, _r)

        def addFilterSet(self, target, _ctx=None):
            return _M_omero.model.Instrument._op_addFilterSet.invoke(self, ((target, ), _ctx))

        def begin_addFilterSet(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addFilterSet.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addFilterSet(self, _r):
            return _M_omero.model.Instrument._op_addFilterSet.end(self, _r)

        def addAllFilterSetSet(self, targets, _ctx=None):
            return _M_omero.model.Instrument._op_addAllFilterSetSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllFilterSetSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addAllFilterSetSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllFilterSetSet(self, _r):
            return _M_omero.model.Instrument._op_addAllFilterSetSet.end(self, _r)

        def removeFilterSet(self, theTarget, _ctx=None):
            return _M_omero.model.Instrument._op_removeFilterSet.invoke(self, ((theTarget, ), _ctx))

        def begin_removeFilterSet(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeFilterSet.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeFilterSet(self, _r):
            return _M_omero.model.Instrument._op_removeFilterSet.end(self, _r)

        def removeAllFilterSetSet(self, targets, _ctx=None):
            return _M_omero.model.Instrument._op_removeAllFilterSetSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllFilterSetSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeAllFilterSetSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllFilterSetSet(self, _r):
            return _M_omero.model.Instrument._op_removeAllFilterSetSet.end(self, _r)

        def clearFilterSet(self, _ctx=None):
            return _M_omero.model.Instrument._op_clearFilterSet.invoke(self, ((), _ctx))

        def begin_clearFilterSet(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_clearFilterSet.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearFilterSet(self, _r):
            return _M_omero.model.Instrument._op_clearFilterSet.end(self, _r)

        def reloadFilterSet(self, toCopy, _ctx=None):
            return _M_omero.model.Instrument._op_reloadFilterSet.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadFilterSet(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_reloadFilterSet.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadFilterSet(self, _r):
            return _M_omero.model.Instrument._op_reloadFilterSet.end(self, _r)

        def unloadOtf(self, _ctx=None):
            return _M_omero.model.Instrument._op_unloadOtf.invoke(self, ((), _ctx))

        def begin_unloadOtf(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_unloadOtf.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadOtf(self, _r):
            return _M_omero.model.Instrument._op_unloadOtf.end(self, _r)

        def sizeOfOtf(self, _ctx=None):
            return _M_omero.model.Instrument._op_sizeOfOtf.invoke(self, ((), _ctx))

        def begin_sizeOfOtf(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_sizeOfOtf.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfOtf(self, _r):
            return _M_omero.model.Instrument._op_sizeOfOtf.end(self, _r)

        def copyOtf(self, _ctx=None):
            return _M_omero.model.Instrument._op_copyOtf.invoke(self, ((), _ctx))

        def begin_copyOtf(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_copyOtf.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyOtf(self, _r):
            return _M_omero.model.Instrument._op_copyOtf.end(self, _r)

        def addOTF(self, target, _ctx=None):
            return _M_omero.model.Instrument._op_addOTF.invoke(self, ((target, ), _ctx))

        def begin_addOTF(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addOTF.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addOTF(self, _r):
            return _M_omero.model.Instrument._op_addOTF.end(self, _r)

        def addAllOTFSet(self, targets, _ctx=None):
            return _M_omero.model.Instrument._op_addAllOTFSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllOTFSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addAllOTFSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllOTFSet(self, _r):
            return _M_omero.model.Instrument._op_addAllOTFSet.end(self, _r)

        def removeOTF(self, theTarget, _ctx=None):
            return _M_omero.model.Instrument._op_removeOTF.invoke(self, ((theTarget, ), _ctx))

        def begin_removeOTF(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeOTF.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeOTF(self, _r):
            return _M_omero.model.Instrument._op_removeOTF.end(self, _r)

        def removeAllOTFSet(self, targets, _ctx=None):
            return _M_omero.model.Instrument._op_removeAllOTFSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllOTFSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeAllOTFSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllOTFSet(self, _r):
            return _M_omero.model.Instrument._op_removeAllOTFSet.end(self, _r)

        def clearOtf(self, _ctx=None):
            return _M_omero.model.Instrument._op_clearOtf.invoke(self, ((), _ctx))

        def begin_clearOtf(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_clearOtf.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearOtf(self, _r):
            return _M_omero.model.Instrument._op_clearOtf.end(self, _r)

        def reloadOtf(self, toCopy, _ctx=None):
            return _M_omero.model.Instrument._op_reloadOtf.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadOtf(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_reloadOtf.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadOtf(self, _r):
            return _M_omero.model.Instrument._op_reloadOtf.end(self, _r)

        def unloadAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Instrument._op_unloadAnnotationLinks.invoke(self, ((), _ctx))

        def begin_unloadAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_unloadAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadAnnotationLinks(self, _r):
            return _M_omero.model.Instrument._op_unloadAnnotationLinks.end(self, _r)

        def sizeOfAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Instrument._op_sizeOfAnnotationLinks.invoke(self, ((), _ctx))

        def begin_sizeOfAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_sizeOfAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfAnnotationLinks(self, _r):
            return _M_omero.model.Instrument._op_sizeOfAnnotationLinks.end(self, _r)

        def copyAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Instrument._op_copyAnnotationLinks.invoke(self, ((), _ctx))

        def begin_copyAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_copyAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyAnnotationLinks(self, _r):
            return _M_omero.model.Instrument._op_copyAnnotationLinks.end(self, _r)

        def addInstrumentAnnotationLink(self, target, _ctx=None):
            return _M_omero.model.Instrument._op_addInstrumentAnnotationLink.invoke(self, ((target, ), _ctx))

        def begin_addInstrumentAnnotationLink(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addInstrumentAnnotationLink.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addInstrumentAnnotationLink(self, _r):
            return _M_omero.model.Instrument._op_addInstrumentAnnotationLink.end(self, _r)

        def addAllInstrumentAnnotationLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Instrument._op_addAllInstrumentAnnotationLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllInstrumentAnnotationLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addAllInstrumentAnnotationLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllInstrumentAnnotationLinkSet(self, _r):
            return _M_omero.model.Instrument._op_addAllInstrumentAnnotationLinkSet.end(self, _r)

        def removeInstrumentAnnotationLink(self, theTarget, _ctx=None):
            return _M_omero.model.Instrument._op_removeInstrumentAnnotationLink.invoke(self, ((theTarget, ), _ctx))

        def begin_removeInstrumentAnnotationLink(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeInstrumentAnnotationLink.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeInstrumentAnnotationLink(self, _r):
            return _M_omero.model.Instrument._op_removeInstrumentAnnotationLink.end(self, _r)

        def removeAllInstrumentAnnotationLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Instrument._op_removeAllInstrumentAnnotationLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllInstrumentAnnotationLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeAllInstrumentAnnotationLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllInstrumentAnnotationLinkSet(self, _r):
            return _M_omero.model.Instrument._op_removeAllInstrumentAnnotationLinkSet.end(self, _r)

        def clearAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Instrument._op_clearAnnotationLinks.invoke(self, ((), _ctx))

        def begin_clearAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_clearAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearAnnotationLinks(self, _r):
            return _M_omero.model.Instrument._op_clearAnnotationLinks.end(self, _r)

        def reloadAnnotationLinks(self, toCopy, _ctx=None):
            return _M_omero.model.Instrument._op_reloadAnnotationLinks.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadAnnotationLinks(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_reloadAnnotationLinks.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadAnnotationLinks(self, _r):
            return _M_omero.model.Instrument._op_reloadAnnotationLinks.end(self, _r)

        def getAnnotationLinksCountPerOwner(self, _ctx=None):
            return _M_omero.model.Instrument._op_getAnnotationLinksCountPerOwner.invoke(self, ((), _ctx))

        def begin_getAnnotationLinksCountPerOwner(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_getAnnotationLinksCountPerOwner.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getAnnotationLinksCountPerOwner(self, _r):
            return _M_omero.model.Instrument._op_getAnnotationLinksCountPerOwner.end(self, _r)

        def linkAnnotation(self, addition, _ctx=None):
            return _M_omero.model.Instrument._op_linkAnnotation.invoke(self, ((addition, ), _ctx))

        def begin_linkAnnotation(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_linkAnnotation.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkAnnotation(self, _r):
            return _M_omero.model.Instrument._op_linkAnnotation.end(self, _r)

        def addInstrumentAnnotationLinkToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Instrument._op_addInstrumentAnnotationLinkToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addInstrumentAnnotationLinkToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_addInstrumentAnnotationLinkToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addInstrumentAnnotationLinkToBoth(self, _r):
            return _M_omero.model.Instrument._op_addInstrumentAnnotationLinkToBoth.end(self, _r)

        def findInstrumentAnnotationLink(self, removal, _ctx=None):
            return _M_omero.model.Instrument._op_findInstrumentAnnotationLink.invoke(self, ((removal, ), _ctx))

        def begin_findInstrumentAnnotationLink(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_findInstrumentAnnotationLink.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findInstrumentAnnotationLink(self, _r):
            return _M_omero.model.Instrument._op_findInstrumentAnnotationLink.end(self, _r)

        def unlinkAnnotation(self, removal, _ctx=None):
            return _M_omero.model.Instrument._op_unlinkAnnotation.invoke(self, ((removal, ), _ctx))

        def begin_unlinkAnnotation(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_unlinkAnnotation.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkAnnotation(self, _r):
            return _M_omero.model.Instrument._op_unlinkAnnotation.end(self, _r)

        def removeInstrumentAnnotationLinkFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Instrument._op_removeInstrumentAnnotationLinkFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeInstrumentAnnotationLinkFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_removeInstrumentAnnotationLinkFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeInstrumentAnnotationLinkFromBoth(self, _r):
            return _M_omero.model.Instrument._op_removeInstrumentAnnotationLinkFromBoth.end(self, _r)

        def linkedAnnotationList(self, _ctx=None):
            return _M_omero.model.Instrument._op_linkedAnnotationList.invoke(self, ((), _ctx))

        def begin_linkedAnnotationList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Instrument._op_linkedAnnotationList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedAnnotationList(self, _r):
            return _M_omero.model.Instrument._op_linkedAnnotationList.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.InstrumentPrx.ice_checkedCast(proxy, '::omero::model::Instrument', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.InstrumentPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::Instrument'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_InstrumentPrx = IcePy.defineProxy('::omero::model::Instrument', InstrumentPrx)

    _M_omero.model._t_Instrument = IcePy.declareClass('::omero::model::Instrument')

    _M_omero.model._t_Instrument = IcePy.defineClass('::omero::model::Instrument', Instrument, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_microscope', (), _M_omero.model._t_Microscope, False, 0),
        ('_detectorSeq', (), _M_omero.model._t_InstrumentDetectorSeq, False, 0),
        ('_detectorLoaded', (), IcePy._t_bool, False, 0),
        ('_objectiveSeq', (), _M_omero.model._t_InstrumentObjectiveSeq, False, 0),
        ('_objectiveLoaded', (), IcePy._t_bool, False, 0),
        ('_lightSourceSeq', (), _M_omero.model._t_InstrumentLightSourceSeq, False, 0),
        ('_lightSourceLoaded', (), IcePy._t_bool, False, 0),
        ('_filterSeq', (), _M_omero.model._t_InstrumentFilterSeq, False, 0),
        ('_filterLoaded', (), IcePy._t_bool, False, 0),
        ('_dichroicSeq', (), _M_omero.model._t_InstrumentDichroicSeq, False, 0),
        ('_dichroicLoaded', (), IcePy._t_bool, False, 0),
        ('_filterSetSeq', (), _M_omero.model._t_InstrumentFilterSetSeq, False, 0),
        ('_filterSetLoaded', (), IcePy._t_bool, False, 0),
        ('_otfSeq', (), _M_omero.model._t_InstrumentOtfSeq, False, 0),
        ('_otfLoaded', (), IcePy._t_bool, False, 0),
        ('_annotationLinksSeq', (), _M_omero.model._t_InstrumentAnnotationLinksSeq, False, 0),
        ('_annotationLinksLoaded', (), IcePy._t_bool, False, 0),
        ('_annotationLinksCountPerOwner', (), _M_omero.sys._t_CountMap, False, 0)
    ))
    Instrument._ice_type = _M_omero.model._t_Instrument

    Instrument._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Instrument._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Instrument._op_getMicroscope = IcePy.Operation('getMicroscope', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_Microscope, False, 0), ())
    Instrument._op_setMicroscope = IcePy.Operation('setMicroscope', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Microscope, False, 0),), (), None, ())
    Instrument._op_unloadDetector = IcePy.Operation('unloadDetector', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Instrument._op_sizeOfDetector = IcePy.Operation('sizeOfDetector', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Instrument._op_copyDetector = IcePy.Operation('copyDetector', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_InstrumentDetectorSeq, False, 0), ())
    Instrument._op_addDetector = IcePy.Operation('addDetector', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Detector, False, 0),), (), None, ())
    Instrument._op_addAllDetectorSet = IcePy.Operation('addAllDetectorSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentDetectorSeq, False, 0),), (), None, ())
    Instrument._op_removeDetector = IcePy.Operation('removeDetector', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Detector, False, 0),), (), None, ())
    Instrument._op_removeAllDetectorSet = IcePy.Operation('removeAllDetectorSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentDetectorSeq, False, 0),), (), None, ())
    Instrument._op_clearDetector = IcePy.Operation('clearDetector', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Instrument._op_reloadDetector = IcePy.Operation('reloadDetector', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Instrument, False, 0),), (), None, ())
    Instrument._op_unloadObjective = IcePy.Operation('unloadObjective', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Instrument._op_sizeOfObjective = IcePy.Operation('sizeOfObjective', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Instrument._op_copyObjective = IcePy.Operation('copyObjective', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_InstrumentObjectiveSeq, False, 0), ())
    Instrument._op_addObjective = IcePy.Operation('addObjective', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Objective, False, 0),), (), None, ())
    Instrument._op_addAllObjectiveSet = IcePy.Operation('addAllObjectiveSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentObjectiveSeq, False, 0),), (), None, ())
    Instrument._op_removeObjective = IcePy.Operation('removeObjective', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Objective, False, 0),), (), None, ())
    Instrument._op_removeAllObjectiveSet = IcePy.Operation('removeAllObjectiveSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentObjectiveSeq, False, 0),), (), None, ())
    Instrument._op_clearObjective = IcePy.Operation('clearObjective', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Instrument._op_reloadObjective = IcePy.Operation('reloadObjective', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Instrument, False, 0),), (), None, ())
    Instrument._op_unloadLightSource = IcePy.Operation('unloadLightSource', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Instrument._op_sizeOfLightSource = IcePy.Operation('sizeOfLightSource', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Instrument._op_copyLightSource = IcePy.Operation('copyLightSource', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_InstrumentLightSourceSeq, False, 0), ())
    Instrument._op_addLightSource = IcePy.Operation('addLightSource', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_LightSource, False, 0),), (), None, ())
    Instrument._op_addAllLightSourceSet = IcePy.Operation('addAllLightSourceSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentLightSourceSeq, False, 0),), (), None, ())
    Instrument._op_removeLightSource = IcePy.Operation('removeLightSource', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_LightSource, False, 0),), (), None, ())
    Instrument._op_removeAllLightSourceSet = IcePy.Operation('removeAllLightSourceSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentLightSourceSeq, False, 0),), (), None, ())
    Instrument._op_clearLightSource = IcePy.Operation('clearLightSource', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Instrument._op_reloadLightSource = IcePy.Operation('reloadLightSource', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Instrument, False, 0),), (), None, ())
    Instrument._op_unloadFilter = IcePy.Operation('unloadFilter', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Instrument._op_sizeOfFilter = IcePy.Operation('sizeOfFilter', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Instrument._op_copyFilter = IcePy.Operation('copyFilter', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_InstrumentFilterSeq, False, 0), ())
    Instrument._op_addFilter = IcePy.Operation('addFilter', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Filter, False, 0),), (), None, ())
    Instrument._op_addAllFilterSet = IcePy.Operation('addAllFilterSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentFilterSeq, False, 0),), (), None, ())
    Instrument._op_removeFilter = IcePy.Operation('removeFilter', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Filter, False, 0),), (), None, ())
    Instrument._op_removeAllFilterSet = IcePy.Operation('removeAllFilterSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentFilterSeq, False, 0),), (), None, ())
    Instrument._op_clearFilter = IcePy.Operation('clearFilter', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Instrument._op_reloadFilter = IcePy.Operation('reloadFilter', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Instrument, False, 0),), (), None, ())
    Instrument._op_unloadDichroic = IcePy.Operation('unloadDichroic', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Instrument._op_sizeOfDichroic = IcePy.Operation('sizeOfDichroic', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Instrument._op_copyDichroic = IcePy.Operation('copyDichroic', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_InstrumentDichroicSeq, False, 0), ())
    Instrument._op_addDichroic = IcePy.Operation('addDichroic', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Dichroic, False, 0),), (), None, ())
    Instrument._op_addAllDichroicSet = IcePy.Operation('addAllDichroicSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentDichroicSeq, False, 0),), (), None, ())
    Instrument._op_removeDichroic = IcePy.Operation('removeDichroic', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Dichroic, False, 0),), (), None, ())
    Instrument._op_removeAllDichroicSet = IcePy.Operation('removeAllDichroicSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentDichroicSeq, False, 0),), (), None, ())
    Instrument._op_clearDichroic = IcePy.Operation('clearDichroic', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Instrument._op_reloadDichroic = IcePy.Operation('reloadDichroic', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Instrument, False, 0),), (), None, ())
    Instrument._op_unloadFilterSet = IcePy.Operation('unloadFilterSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Instrument._op_sizeOfFilterSet = IcePy.Operation('sizeOfFilterSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Instrument._op_copyFilterSet = IcePy.Operation('copyFilterSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_InstrumentFilterSetSeq, False, 0), ())
    Instrument._op_addFilterSet = IcePy.Operation('addFilterSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilterSet, False, 0),), (), None, ())
    Instrument._op_addAllFilterSetSet = IcePy.Operation('addAllFilterSetSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentFilterSetSeq, False, 0),), (), None, ())
    Instrument._op_removeFilterSet = IcePy.Operation('removeFilterSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_FilterSet, False, 0),), (), None, ())
    Instrument._op_removeAllFilterSetSet = IcePy.Operation('removeAllFilterSetSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentFilterSetSeq, False, 0),), (), None, ())
    Instrument._op_clearFilterSet = IcePy.Operation('clearFilterSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Instrument._op_reloadFilterSet = IcePy.Operation('reloadFilterSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Instrument, False, 0),), (), None, ())
    Instrument._op_unloadOtf = IcePy.Operation('unloadOtf', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Instrument._op_sizeOfOtf = IcePy.Operation('sizeOfOtf', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Instrument._op_copyOtf = IcePy.Operation('copyOtf', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_InstrumentOtfSeq, False, 0), ())
    Instrument._op_addOTF = IcePy.Operation('addOTF', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_OTF, False, 0),), (), None, ())
    Instrument._op_addAllOTFSet = IcePy.Operation('addAllOTFSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentOtfSeq, False, 0),), (), None, ())
    Instrument._op_removeOTF = IcePy.Operation('removeOTF', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_OTF, False, 0),), (), None, ())
    Instrument._op_removeAllOTFSet = IcePy.Operation('removeAllOTFSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentOtfSeq, False, 0),), (), None, ())
    Instrument._op_clearOtf = IcePy.Operation('clearOtf', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Instrument._op_reloadOtf = IcePy.Operation('reloadOtf', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Instrument, False, 0),), (), None, ())
    Instrument._op_unloadAnnotationLinks = IcePy.Operation('unloadAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Instrument._op_sizeOfAnnotationLinks = IcePy.Operation('sizeOfAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Instrument._op_copyAnnotationLinks = IcePy.Operation('copyAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_InstrumentAnnotationLinksSeq, False, 0), ())
    Instrument._op_addInstrumentAnnotationLink = IcePy.Operation('addInstrumentAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentAnnotationLink, False, 0),), (), None, ())
    Instrument._op_addAllInstrumentAnnotationLinkSet = IcePy.Operation('addAllInstrumentAnnotationLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentAnnotationLinksSeq, False, 0),), (), None, ())
    Instrument._op_removeInstrumentAnnotationLink = IcePy.Operation('removeInstrumentAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentAnnotationLink, False, 0),), (), None, ())
    Instrument._op_removeAllInstrumentAnnotationLinkSet = IcePy.Operation('removeAllInstrumentAnnotationLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentAnnotationLinksSeq, False, 0),), (), None, ())
    Instrument._op_clearAnnotationLinks = IcePy.Operation('clearAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Instrument._op_reloadAnnotationLinks = IcePy.Operation('reloadAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Instrument, False, 0),), (), None, ())
    Instrument._op_getAnnotationLinksCountPerOwner = IcePy.Operation('getAnnotationLinksCountPerOwner', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.sys._t_CountMap, False, 0), ())
    Instrument._op_linkAnnotation = IcePy.Operation('linkAnnotation', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), ((), _M_omero.model._t_InstrumentAnnotationLink, False, 0), ())
    Instrument._op_addInstrumentAnnotationLinkToBoth = IcePy.Operation('addInstrumentAnnotationLinkToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentAnnotationLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Instrument._op_findInstrumentAnnotationLink = IcePy.Operation('findInstrumentAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), ((), _M_omero.model._t_InstrumentAnnotationLinksSeq, False, 0), ())
    Instrument._op_unlinkAnnotation = IcePy.Operation('unlinkAnnotation', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), None, ())
    Instrument._op_removeInstrumentAnnotationLinkFromBoth = IcePy.Operation('removeInstrumentAnnotationLinkFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_InstrumentAnnotationLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Instrument._op_linkedAnnotationList = IcePy.Operation('linkedAnnotationList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_InstrumentLinkedAnnotationSeq, False, 0), ())

    _M_omero.model.Instrument = Instrument
    del Instrument

    _M_omero.model.InstrumentPrx = InstrumentPrx
    del InstrumentPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero
